function par = computeFullBodyMassParams(par)
    % Center of Mass Locations
    % Average Male locations
    if par.Gender == 'm'
        par.CoM_UppTrunk = (1-0.2999)*par.Len_UppTrunk;
        par.CoM_MidTrunk = (1-0.4502)*par.Len_MidTrunk;
        par.CoM_LowTrunk = (1-0.6115)*par.Len_LowTrunk;
        par.CoM_Thigh = 0.4095*par.Len_Thigh;
        par.CoM_Shank = 0.4495*par.Len_Shank;
        par.CoM_Foot_x = 0.4415*par.Len_Foot;
        par.CoM_Foot_z = 0.5*par.Height_Foot;
%         par.CoM_Foot_z = 0.75*(0.5*(par.Height_Foot_a+par.Height_Foot_b));
        par.CoM_UppArm = 0.5772*par.Len_UppArm;
        par.CoM_LowArm = 0.4574*par.Len_LowArm;
    else
        % Average Female locations
        par.CoM_UppTrunk = (1-0.2077)*par.Len_UppTrunk;
        par.CoM_MidTrunk = (1-0.4512)*par.Len_MidTrunk;
        par.CoM_LowTrunk = (1-0.4920)*par.Len_LowTrunk;
        par.CoM_Thigh = 0.3612*par.Len_Thigh;
        par.CoM_Shank = 0.4416*par.Len_Shank;
        par.CoM_Foot_x = 0.4014*par.Len_Foot;
        par.CoM_Foot_z = 0.5*par.Height_Foot;
        par.CoM_UppArm = 0.5754*par.Len_UppArm;
        par.CoM_LowArm = 0.4559*par.Len_LowArm;
    end

    % Part Masses
    if par.Gender == 'm'
        % Average mass as a % of total mass for males
        par.Mass_UppTrunk = 0.1596;
        par.Mass_MidTrunk = 0.1633;
        par.Mass_LowTrunk = 0.1117;
        par.Mass_Thigh = 0.1416;
        par.Mass_Shank = 0.0433;
        par.Mass_Foot = 0.0137;
        par.Mass_UppArm = 0.0271;
        par.Mass_LowArm = 0.0162;
    else
        % Average mass as a % of total mass for females
        par.Mass_UppTrunk = 0.1545;
        par.Mass_MidTrunk = 0.1465;
        par.Mass_LowTrunk = 0.1247;
        par.Mass_Thigh = 0.1478;
        par.Mass_Shank = 0.0481;
        par.Mass_Foot = 0.0129;
        par.Mass_UppArm = 0.0255;
        par.Mass_LowArm = 0.0138;
    end

    % Moments of inertia
    %  With the exception of the foot the following holds:
    %   - x is the transverse axis
    %   - y is the sagittal axis
    %   - z is the longitudinal axis
    %  In the case of the foot the x and z axis are swapped.
    if par.Gender == 'm'
        % Average moments for males
        par.Moment_UppTrunk_x = par.Mass_UppTrunk*par.Len_UppTrunk^2*0.454^2;
        par.Moment_MidTrunk_x = par.Mass_MidTrunk*par.Len_MidTrunk^2*0.383^2;
        par.Moment_LowTrunk_x = par.Mass_LowTrunk*par.Len_LowTrunk^2*0.551^2;
        par.Moment_Thigh_x = par.Mass_Thigh*par.Len_Thigh^2*0.329^2;
        par.Moment_Shank_x = par.Mass_Shank*par.Len_Shank^2*0.249^2;
        par.Moment_Foot_z = par.Mass_Foot*par.Len_Foot^2*0.245^2;
        par.Moment_UppArm_x = par.Mass_UppArm*par.Len_UppArm^2*0.269^2;
        par.Moment_LowArm_x = par.Mass_LowArm*par.Len_LowArm^2*0.265^2;

        par.Moment_UppTrunk_y = par.Mass_UppTrunk*par.Len_UppTrunk^2*0.716^2;
        par.Moment_MidTrunk_y = par.Mass_MidTrunk*par.Len_MidTrunk^2*0.482^2;
        par.Moment_LowTrunk_y = par.Mass_LowTrunk*par.Len_LowTrunk^2*0.615^2;
        par.Moment_Thigh_y = par.Mass_Thigh*par.Len_Thigh^2*0.329^2;
        par.Moment_Shank_y = par.Mass_Shank*par.Len_Shank^2*0.255^2;
        par.Moment_Foot_y = par.Mass_Foot*par.Len_Foot^2*0.257^2;
        par.Moment_UppArm_y = par.Mass_UppArm*par.Len_UppArm^2*0.285^2;
        par.Moment_LowArm_y = par.Mass_LowArm*par.Len_LowArm^2*0.276^2;

        par.Moment_UppTrunk_z = par.Mass_UppTrunk*par.Len_UppTrunk^2*0.659^2;
        par.Moment_MidTrunk_z = par.Mass_MidTrunk*par.Len_MidTrunk^2*0.468^2;
        par.Moment_LowTrunk_z = par.Mass_LowTrunk*par.Len_LowTrunk^2*0.587^2;
        par.Moment_Thigh_z = par.Mass_Thigh*par.Len_Thigh^2*0.149^2;
        par.Moment_Shank_z = par.Mass_Shank*par.Len_Shank^2*0.103^2;
        par.Moment_Foot_x = par.Mass_Foot*par.Len_Foot^2*0.124^2;
        par.Moment_UppArm_z = par.Mass_UppArm*par.Len_UppArm^2*0.158^2;
        par.Moment_LowArm_z = par.Mass_LowArm*par.Len_LowArm^2*0.121^2;
    else
        % Average moments for females
        par.Moment_UppTrunk_x = par.Mass_UppTrunk*par.Len_UppTrunk^2*0.502^2;
        par.Moment_MidTrunk_x = par.Mass_MidTrunk*par.Len_MidTrunk^2*0.354^2;
        par.Moment_LowTrunk_x = par.Mass_LowTrunk*par.Len_LowTrunk^2*0.402^2;
        par.Moment_Thigh_x = par.Mass_Thigh*par.Len_Thigh^2*0.364^2;
        par.Moment_Shank_x = par.Mass_Shank*par.Len_Shank^2*0.267^2;
        par.Moment_Foot_z = par.Mass_Foot*par.Len_Foot^2*0.279^2;
        par.Moment_UppArm_x = par.Mass_UppArm*par.Len_UppArm^2*0.260^2;
        par.Moment_LowArm_x = par.Mass_LowArm*par.Len_LowArm^2*0.257^2;

        par.Moment_UppTrunk_y = par.Mass_UppTrunk*par.Len_UppTrunk^2*0.746^2;
        par.Moment_MidTrunk_y = par.Mass_MidTrunk*par.Len_MidTrunk^2*0.433^2;
        par.Moment_LowTrunk_y = par.Mass_LowTrunk*par.Len_LowTrunk^2*0.433^2;
        par.Moment_Thigh_y = par.Mass_Thigh*par.Len_Thigh^2*0.369^2;
        par.Moment_Shank_y = par.Mass_Shank*par.Len_Shank^2*0.271^2;
        par.Moment_Foot_y = par.Mass_Foot*par.Len_Foot^2*0.299^2;
        par.Moment_UppArm_y = par.Mass_UppArm*par.Len_UppArm^2*0.278^2;
        par.Moment_LowArm_y = par.Mass_LowArm*par.Len_LowArm^2*0.261^2;

        par.Moment_UppTrunk_z = par.Mass_UppTrunk*par.Len_UppTrunk^2*0.718^2;
        par.Moment_MidTrunk_z = par.Mass_MidTrunk*par.Len_MidTrunk^2*0.415^2;
        par.Moment_LowTrunk_z = par.Mass_LowTrunk*par.Len_LowTrunk^2*0.444^2;
        par.Moment_Thigh_z = par.Mass_Thigh*par.Len_Thigh^2*0.162^2;
        par.Moment_Shank_z = par.Mass_Shank*par.Len_Shank^2*0.093^2;
        par.Moment_Foot_x = par.Mass_Foot*par.Len_Foot^2*0.139^2;
        par.Moment_UppArm_z = par.Mass_UppArm*par.Len_UppArm^2*0.148^2;
        par.Moment_LowArm_z = par.Mass_LowArm*par.Len_LowArm^2*0.094^2;
    end
end
