function [cOrient, cCoM, dcOrientdq, dcCoMdq, d2cOrientd2q, d2cCoMd2q] = ...
                computeHeirarchicalPartInfo(pOrient, pCoM, ...
                                            pJoint, cJoint, cRelOrient, ...
                                            dpOrientdq, dpCoMdq, ...
                                            dpJointdq, dcJointdq,  dcRelOrientdq, ...
                                            d2pOrientd2q, d2pCoMd2q, ...
                                            d2pJointd2q, d2cJointd2q, d2cRelOrientd2q)

    persistent fooMat dQdq dQCdq
    if isempty(fooMat)
        dQdq = quatprematderiv;
        dQCdq = quatpostmatderiv;
        fooMat = zeros(16,12);
        for i = 1:4
            mat = 2*dQdq(:,:,i)*dQCdq(:,:,i).';
            i1 = (i-1)*4+i;
            fooMat(i1,:) = reshape(mat(2:4,:),[1,12]);
            for j = (i+1):4
                mat = (dQdq(:,:,i)*dQCdq(:,:,j).' + dQdq(:,:,j)*dQCdq(:,:,i).');
                i1 = (i-1)*4+j;
                i2 = (j-1)*4+i;
                fooMat(i1,:) = reshape(mat(2:4,:),[1,12]);
                fooMat(i2,:) = fooMat(i1,:);
            end
        end
        fooMat = reshape(fooMat,[16*3,4]);
    end

    % Compute orientation and center of mass
    cOrient = quatmult(pOrient,cRelOrient);
    pR = quatrot(pOrient);
    cR = quatrot(cOrient);
    cCoM = pCoM + pR*pJoint + cR*cJoint;

    % Compute first derivatives
    if nargout > 2
        pQ = quatpremat(pOrient);
        pQC = quatpostmat(pOrient);
        cQ = quatpremat(cOrient);
        cQC = quatpostmat(cOrient);

        qDims = size(dpCoMdq,2);
        qDims2 = qDims^2;

        if issparse(dpOrientdq)
            nzCols = find(any(dpOrientdq));
            dcOrientdq = sparse([],[],[],4,qDims,4*length(nzCols));

            dcOrientdq(:,nzCols) = quatmult(full(dpOrientdq(:,nzCols)),cRelOrient);
        else
            dcOrientdq = quatmult(dpOrientdq,cRelOrient);
        end
        if issparse(dcRelOrientdq)
            nzCols = find(any(dcRelOrientdq));
            dcOrientdq(:,nzCols) = dcOrientdq(:,nzCols) + quatmult(pOrient,full(dcRelOrientdq(:,nzCols)));
        else
            dcOrientdq = dcOrientdq + quatmult(pOrient,dcRelOrientdq);
        end

        dcCoMdq = dpCoMdq;
        if issparse(dpJointdq)
            nzCols = find(any(dpJointdq));
            dcCoMdq(:,nzCols) = dcCoMdq(:,nzCols) + pR*dpJointdq(:,nzCols);
        else
            dcCoMdq = dcCoMdq + pR*dpJointdq;
        end
        if issparse(dcJointdq)
            nzCols = find(any(dcJointdq));
            dcCoMdq(:,nzCols) = dcCoMdq(:,nzCols) + cR*dcJointdq(:,nzCols);
        else
            dcCoMdq = dcCoMdq + cR*dcJointdq;
        end

        pMat = zeros([3,4]);
        cMat = zeros([3,4]);
        for j = 1:4
            pMat(:,j) = (dQdq(2:4,:,j)*pQC(2:4,:).' + pQ(2:4,:)*dQCdq(2:4,:,j).')*pJoint;
            cMat(:,j) = (dQdq(2:4,:,j)*cQC(2:4,:).' + cQ(2:4,:)*dQCdq(2:4,:,j).')*cJoint;
        end

        if issparse(dpOrientdq)
            nzCols = find(any(dpOrientdq));
            dcCoMdq(:,nzCols) = dcCoMdq(:,nzCols) + pMat*dpOrientdq(:,nzCols);
        else
            dcCoMdq = dcCoMdq + pMat*dpOrientdq;
        end
        if issparse(dcOrientdq)
            nzCols = find(any(dcOrientdq));
            dcCoMdq(:,nzCols) = dcCoMdq(:,nzCols) + cMat*dcOrientdq(:,nzCols);
        else
            dcCoMdq = dcCoMdq + cMat*dcOrientdq;
        end
    end
    
    % Compute second derivatives
    if nargout > 4
        if issparse(d2pOrientd2q)
            nzCols = find(any(d2pOrientd2q));

            d2cOrientd2q = sparse([],[],[],4,qDims2,4*length(nzCols));
            d2cOrientd2q(:,nzCols) = d2cOrientd2q(:,nzCols) + quatmult(full(d2pOrientd2q(:,nzCols)),cRelOrient);
        else
            d2cOrientd2q = quatmult(d2pOrientd2q,cRelOrient);
        end

        if issparse(d2cRelOrientd2q)
            nzCols = find(any(d2cRelOrientd2q));
            d2cOrientd2q(:,nzCols) = d2cOrientd2q(:,nzCols) + quatmult(pOrient,full(d2cRelOrientd2q(:,nzCols)));
        else
            d2cOrientd2q = d2cOrientd2q + quatmult(pOrient,d2cRelOrientd2q);
        end

        if issparse(dpOrientdq) || issparse(dcRelOrientdq)
            nzpCols = find(any(dpOrientdq));
            nzcCols = find(any(dcRelOrientdq));
            upd = quatmult(full(dpOrientdq(:,nzpCols)),full(dcRelOrientdq(:,nzcCols)));
            if ~isempty(nzpCols)
                for k = 1:length(nzcCols)
                    mj = nzcCols(k);
                    j = (nzpCols-1)*qDims + mj;
                    d2cOrientd2q(:,j) = d2cOrientd2q(:,j) + upd(:,:,k);
                    j = (mj-1)*qDims + nzpCols;
                    d2cOrientd2q(:,j) = d2cOrientd2q(:,j) + upd(:,:,k);
                end
            end
        else
            for j = 1:qDims2
                cj = ceil(j/qDims);
                mj = mod(j-1,qDims)+1;

                d2cOrientd2q(:,j) = d2cOrientd2q(:,j) + quatmult(dpOrientdq(:,cj),dcRelOrientdq(:,mj)) + quatmult(dpOrientdq(:,mj),dcRelOrientdq(:,cj));
            end
        end

        d2cCoMd2q = d2pCoMd2q;
        % 4
        if issparse(d2pJointd2q)
            nzCols = find(any(d2pJointd2q));
            d2cCoMd2q(:,nzCols) = d2cCoMd2q(:,nzCols) + pR*d2pJointd2q(:,nzCols);
        else
            d2cCoMd2q = d2cCoMd2q + reshape(pR*reshape(d2pJointd2q,[3,qDims2]),[3,qDims,qDims]);
        end
        if issparse(d2cJointd2q)
            nzCols = find(any(d2cJointd2q));
            d2cCoMd2q(:,nzCols) = d2cCoMd2q(:,nzCols) + cR*d2cJointd2q(:,nzCols);
        else
            d2cCoMd2q = d2cCoMd2q + reshape(cR*reshape(d2cJointd2q,[3,qDims2]),[3,qDims,qDims]);
        end
        
        % 3 (== 0)
        
        % 2
        if issparse(d2pOrientd2q)
            nzCols = find(any(d2pOrientd2q));
            d2cCoMd2q(:,nzCols) = d2cCoMd2q(:,nzCols) + pMat*d2pOrientd2q(:,nzCols);
        else
            d2cCoMd2q = d2cCoMd2q + reshape(pMat*reshape(d2pOrientd2q,[4,qDims2]),[3,qDims,qDims]);
        end
        if issparse(d2cOrientd2q)
            nzCols = find(any(d2cOrientd2q));
            d2cCoMd2q(:,nzCols) = d2cCoMd2q(:,nzCols) + cMat*d2cOrientd2q(:,nzCols);
        else
            d2cCoMd2q = d2cCoMd2q + reshape(cMat*reshape(d2cOrientd2q,[4,qDims2]),[3,qDims,qDims]);
        end

        % 1
        % dpOrientdq and especially dcOrientdq are very sparse.  There
        % should be a way to take advantage of this but what I've tried
        % below is actually about 50% slower.  There may not be anything
        % else to do.
        fooP = reshape(fooMat*[0;pJoint],[4,4,3]);
%         if issparse(dpOrientdq)
%             nzpCols = find(any(dpOrientdq));
%             tmpd2 = sparse([],[],[],qDims,qDims,length(nzpCols)^2);
%             for i = 1:3
%                 tmpd2(nzpCols,nzpCols) = dpOrientdq(:,nzpCols).'*fooP(:,:,i)*dpOrientdq(:,nzpCols);
%                 d2cCoMd2q(i,:) = d2cCoMd2q(i,:) + reshape(tmpd2,[1,qDims2]);
%             end
%         else
            for i = 1:3
                d2cCoMd2q(i,:) = d2cCoMd2q(i,:) + reshape(dpOrientdq.'*fooP(:,:,i)*dpOrientdq,[1,qDims2]);
            end
%         end
        fooC = reshape(fooMat*[0;cJoint],[4,4,3]);
%         if issparse(dcOrientdq)
%             nzcCols = find(any(dcOrientdq));
%             tmpd2 = sparse([],[],[],qDims,qDims,length(nzcCols)^2);
%             for i = 1:3
%                 tmpd2(nzcCols,nzcCols) = dcOrientdq(:,nzcCols).'*fooC(:,:,i)*dcOrientdq(:,nzcCols);
%                 d2cCoMd2q(i,:) = d2cCoMd2q(i,:) + reshape(tmpd2,[1,qDims2]);
%             end
%         else
            for i = 1:3
                d2cCoMd2q(i,:) = d2cCoMd2q(i,:) + reshape(dcOrientdq.'*fooC(:,:,i)*dcOrientdq,[1,qDims2]);
            end
%         end
    end
end
